#!/bin/bash
#
# Copyright (C) 2010 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#  This shell script is used to rebuild the gcc and toolchain binaries
#  for the Android NDK.
#

# include common function and variable definitions
. `dirname $0`/prebuilt-common.sh

PROGRAM_PARAMETERS="<src-dir> <ndk-dir> <toolchain>"

PROGRAM_DESCRIPTION=\
"Rebuild the gcc toolchain prebuilt binaries for the Android NDK.

Where <src-dir> is the location of toolchain sources, <ndk-dir> is
the top-level NDK installation path and <toolchain> is the name of
the toolchain to use (e.g. arm-linux-androideabi-4.4.3)."

RELEASE=`date +%Y%m%d`
BUILD_OUT=/tmp/ndk-$USER/build/toolchain
OPTION_BUILD_OUT=
register_var_option "--build-out=<path>" OPTION_BUILD_OUT "Set temporary build directory"

# Note: platform API level 9 or higher is needed for proper C++ support
PLATFORM=android-9
register_var_option "--platform=<name>"  PLATFORM "Specify platform name"

OPTION_SYSROOT=
register_var_option "--sysroot=<path>"   OPTION_SYSROOT   "Specify sysroot directory directly"

GDB_VERSION=7.2.50.20110211
register_var_option "--gdb-version=<version>" GDB_VERSION "Specify gdb version"

GCC_PATH=gcc
register_var_option "--gcc-path=<path>" GCC_PATH "Use specific gcc path"

GDB_PATH=
register_var_option "--gdb-path=<path>" GDB_PATH "Use specific gdb path"

GDB_WITH_PYTHON=
register_var_option "--gdb-with-python=<path-to-python>" GDB_WITH_PYTHON "Specify which python to build into gdb"

BINUTILS_VERSION=2.20.1
register_var_option "--binutils-version=<version>" BINUTILS_VERSION "Specify binutils version"

GMP_VERSION=4.3.2
register_var_option "--gmp-version=<version>" GMP_VERSION "Specify gmp version"

MPFR_VERSION=2.4.2
register_var_option "--mpfr-version=<version>" MPFR_VERSION "Specify mpfr version"

JOBS=$TC_BUILD_NUM_CPUS
register_var_option "-j<number>" JOBS "Use <number> parallel build jobs"

COPY_LIBSTDCXX=no
register_var_option "--copy-libstdcxx" COPY_LIBSTDCXX "Copy libstdc++ to <ndk-dir>/$GNUSTL_SUBDIR"

KEEP_LIBSTDCXX=no
register_var_option "--keep-libstdcxx" KEEP_LIBSTDCXX "Experimental: keep libstdc++ inside toolchain"

register_mingw_option
register_try64_option

extract_parameters "$@"

fix_option BUILD_OUT "$OPTION_BUILD_OUT" "build directory"
setup_default_log_file $BUILD_OUT/config.log

set_parameters ()
{
    SRC_DIR="$1"
    NDK_DIR="$2"
    TOOLCHAIN="$3"

    # Check source directory
    #
    if [ -z "$SRC_DIR" ] ; then
        echo "ERROR: Missing source directory parameter. See --help for details."
        exit 1
    fi

    if [ ! -d "$SRC_DIR/gcc" ] ; then
        echo "ERROR: Source directory does not contain gcc sources: $SRC_DIR"
        exit 1
    fi

    log "Using source directory: $SRC_DIR"

    # Check NDK installation directory
    #
    if [ -z "$NDK_DIR" ] ; then
        echo "ERROR: Missing NDK directory parameter. See --help for details."
        exit 1
    fi

    if [ ! -d "$NDK_DIR" ] ; then
        mkdir -p $NDK_DIR
        if [ $? != 0 ] ; then
            echo "ERROR: Could not create target NDK installation path: $NDK_DIR"
            exit 1
        fi
    fi

    log "Using NDK directory: $NDK_DIR"

    # Check toolchain name
    #
    if [ -z "$TOOLCHAIN" ] ; then
        echo "ERROR: Missing toolchain name parameter. See --help for details."
        exit 1
    fi
}

set_parameters $PARAMETERS

prepare_host_flags

parse_toolchain_name

fix_sysroot "$OPTION_SYSROOT"

if [ -z $GDB_PATH ] ; then
    GDB_PATH=$SRC_DIR/gdb/gdb-$GDB_VERSION
fi

if [ ! -d $GDB_PATH ] ; then
    echo "ERROR: Missing gdb sources: $GDB_PATH"
    echo "       Use --gdb-version=<version> or --gdb-path=<path> to specify alternative."
    exit 1
fi

fix_option BINUTILS_VERSION "$OPTION_BINUTILS_VERSION" "binutils version"
if [ ! -d $SRC_DIR/binutils/binutils-$BINUTILS_VERSION ] ; then
    echo "ERROR: Missing binutils sources: $SRC_DIR/binutils/binutils-$BINUTILS_VERSION"
    echo "       Use --binutils-version=<version> to specify alternative."
    exit 1
fi

fix_option MPFR_VERSION "$OPTION_MPFR_VERSION" "mpfr version"
if [ ! -f $SRC_DIR/mpfr/mpfr-$MPFR_VERSION.tar.bz2 ] ; then
    echo "ERROR: Missing mpfr sources: $SRC_DIR/mpfr/mpfr-$MPFR_VERSION.tar.bz2"
    echo "       Use --mpfr-version=<version> to specify alternative."
    exit 1
fi

set_toolchain_ndk $NDK_DIR $TOOLCHAIN

dump "Using C compiler: $CC"
dump "Using C++ compiler: $CXX"

fix_cxxstl_vars $GCC_VERSION

# Location where the toolchain license files are
TOOLCHAIN_LICENSES=$ANDROID_NDK_ROOT/build/tools/toolchain-licenses

# Copy the sysroot to the installation prefix. This prevents the generated
# binaries from containing hard-coding host paths
TOOLCHAIN_SYSROOT=$TOOLCHAIN_PATH/sysroot
dump "Sysroot  : Copying: $SYSROOT --> $TOOLCHAIN_SYSROOT"
mkdir -p $TOOLCHAIN_SYSROOT && (cd $SYSROOT && tar ch *) | (cd $TOOLCHAIN_SYSROOT && tar x)
if [ $? != 0 ] ; then
    echo "Error while copying sysroot files. See $TMPLOG"
    exit 1
fi

# For x86, we currently need to force the usage of Android-specific C runtime
# object files to generate a few target binaries. Ideally, this should be directly
# handled by the GCC configuration scripts, just like with ARM.
#
if [ "$ARCH" = "x86" ]; then
    ABI_LDFLAGS_FOR_TARGET=" -nostartfiles $TOOLCHAIN_SYSROOT/usr/lib/crtbegin_dynamic.o $TOOLCHAIN_SYSROOT/usr/lib/crtend_android.o"
    dump "Forcing -nostartfiles: $ABI_LDFLAGS_FOR_TARGET"
fi

# configure the toolchain
#
dump "Configure: $TOOLCHAIN toolchain build"
# Old versions of the toolchain source packages placed the
# configure script at the top-level. Newer ones place it under
# the build directory though. Probe the file system to check
# this.
BUILD_SRCDIR=$SRC_DIR/build
if [ ! -d $BUILD_SRCDIR ] ; then
    BUILD_SRCDIR=$SRC_DIR
fi
# rm -rf $BUILD_OUT
OLD_ABI="${ABI}"
export CC CXX
export CFLAGS_FOR_TARGET="$ABI_CFLAGS_FOR_TARGET"
export CXXFLAGS_FOR_TARGET="$ABI_CXXFLAGS_FOR_TARGET"
export LDFLAGS_FOR_TARGET="$ABI_LDFLAGS_FOR_TARGET"
# Needed to build a 32-bit gmp on 64-bit systems
export ABI=$HOST_GMP_ABI
# -Wno-error is needed because our gdb-6.6 sources use -Werror by default
# and fail to build with recent GCC versions.
export CFLAGS="-Wno-error $HOST_CFLAGS"
export LDFLAGS="$HOST_LDFLAGS"
if [ -d $GDB_PATH ] ; then
    ln -s $GDB_PATH $SRC_DIR/gdb/gdb-$GDB_VERSION
fi

if [ -d $GCC_PATH ] ; then
    ln -s $GCC_PATH $SRC_DIR/gcc/gcc-$GCC_VERSION
fi

mkdir -p $BUILD_OUT
pushd $BUILD_OUT
cd ..
mkdir install
cd install
EXPAT_INSTALL=`pwd`
popd
pushd $SRC_DIR/gdb
OS=${OSTYPE//[0-9.]/}
if [ "$OS" = "darwin" ] ; then
#    curl --insecure -S -L -O http://downloads.sourceforge.net/sourceforge/expat/expat-2.0.1.tar.gz
     curl --insecure -S -L -O http://fossies.org/unix/www/expat-2.0.1.tar.gz
else
#    wget -c http://downloads.sourceforge.net/sourceforge/expat/expat-2.0.1.tar.gz
     wget -c http://fossies.org/unix/www/expat-2.0.1.tar.gz
fi
if [ ! -d expat-2.0.1 ]
then
    tar xzvf expat-2.0.1.tar.gz
fi
pushd expat-2.0.1
    ./configure --disable-shared --enable-static -prefix=$EXPAT_INSTALL
    make
    make install
popd
popd

if [ "$GDB_WITH_PYTHON" != "" ] ; then
    WITH_PYTHON=--with-python=$GDB_WITH_PYTHON
fi

if [ "$GCC_VERSION" = "4.4.3" ] ; then
    WITH_GRAPHITE="--with-ppl-version=0.10.2 --with-cloog-version=0.15.11 --enable-graphite=yes"
else
    WITH_GRAPHITE="--with-ppl-version=0.11.2 --with-cloog-version=0.16.3 --with-mpc-version=0.9 --enable-graphite=yes"
fi

if [ ! -f $BUILD_SRCDIR/Makefile ] ; then
cd $BUILD_OUT && run \
$BUILD_SRCDIR/configure --target=$ABI_CONFIGURE_TARGET \
                        --enable-initfini-array \
                        --host=$ABI_CONFIGURE_HOST \
                        --build=$ABI_CONFIGURE_BUILD \
                        --disable-nls \
                        --prefix=$TOOLCHAIN_PATH \
                        --with-sysroot=$TOOLCHAIN_SYSROOT \
                        --with-binutils-version=$BINUTILS_VERSION \
                        --with-mpfr-version=$MPFR_VERSION \
                        --with-gmp-version=$GMP_VERSION \
                        --with-gcc-version=$GCC_VERSION \
                        --with-gdb-version=$GDB_VERSION \
                        $WITH_GRAPHITE \
                        --with-expat=yes \
                        --with-libexpat-prefix=$EXPAT_INSTALL \
                        $WITH_PYTHON \
                        $ABI_CONFIGURE_EXTRA_FLAGS
if [ $? != 0 ] ; then
    dump "Error while trying to configure toolchain build. See $TMPLOG"
    exit 1
fi
fi
ABI="$OLD_ABI"
# build the toolchain
dump "Building : $TOOLCHAIN toolchain [this can take a long time]."
cd $BUILD_OUT &&
export CC CXX &&
export ABI=$HOST_GMP_ABI &&
run make install-host-cloog -j9
run make -j9
RESULT=$?
# if [ "$OSTYPE" != "msys" -a $RESULT != 0 ] ; then
#    echo "Error while building toolchain. See $TMPLOG"
#    exit 1
# fi
dump "Build result : $RESULT"
# install the toolchain to its final location
dump "Install  : $TOOLCHAIN toolchain binaries."
cd $BUILD_OUT && run make install
INSTALL_RESULT=$?
# if [ "$OSTYPE" != "msys" -a $INSTALL_RESULT != 0 ] ; then
#    echo "Error while installing toolchain. See $TMPLOG"
#    exit 1
# fi

# if [ "$OSTYPE" = "msys" ] ; then
    while [ $RESULT != 0 -o $INSTALL_RESULT != 0 ]
    do
        cd $BUILD_OUT && rm stmp-build-target-gcc &&
        export CC CXX &&
        export ABI=$HOST_GMP_ABI &&
        run make -j1
        RESULT=$?
        cd $BUILD_OUT && run make install
        INSTALL_RESULT=$?
        dump "Build result (r) : make: $RESULT, make install: $INSTALL_RESULT"
    done
# fi

ABI="$OLD_ABI"

# don't forget to copy the GPL and LGPL license files
run cp -f $TOOLCHAIN_LICENSES/COPYING $TOOLCHAIN_LICENSES/COPYING.LIB $TOOLCHAIN_PATH

# remove some unneeded files
run rm -f $TOOLCHAIN_PATH/bin/*-gccbug
run rm -rf $TOOLCHAIN_PATH/info
run rm -rf $TOOLCHAIN_PATH/man
run rm -rf $TOOLCHAIN_PATH/share
run rm -rf $TOOLCHAIN_PATH/lib/gcc/$ABI_CONFIGURE_TARGET/*/install-tools
run rm -rf $TOOLCHAIN_PATH/lib/gcc/$ABI_CONFIGURE_TARGET/*/plugin
run rm -rf $TOOLCHAIN_PATH/libexec/gcc/$ABI_CONFIGURE_TARGET/*/install-tools
run rm -rf $TOOLCHAIN_PATH/lib32/libiberty.a
run rm -rf $TOOLCHAIN_PATH/$ABI_CONFIGURE_TARGET/lib/libiberty.a
run rm -rf $TOOLCHAIN_PATH/$ABI_CONFIGURE_TARGET/lib/*/libiberty.a

# Copy libstdc++ headers and libraries if needed
if [ "$COPY_LIBSTDCXX" = "yes" ] ; then
    dump "Copying libstdc++ prebuilt binaries."
    mkdir -p $BUILD_OUT/libstdc++/$TOOLCHAIN
    $ANDROID_NDK_ROOT/build/tools/copy-libstdcxx.sh "$TOOLCHAIN_PATH" "$NDK_DIR" "$GCC_VERSION" --toolchain=$TOOLCHAIN --out-dir=$BUILD_OUT/libstdc++/$TOOLCHAIN
    $ANDROID_NDK_ROOT/build/tools/copy-libstdcxx.sh "$TOOLCHAIN_PATH" "$NDK_DIR" "$GCC_VERSION" --toolchain=$TOOLCHAIN
fi

# Remove libstdc++ for now (will add it differently later)
# We had to build it to get libsupc++ which we keep.
if [ "$KEEP_LIBSTDCXX" = "no" ] ; then
    run rm -rf $TOOLCHAIN_PATH/$ABI_CONFIGURE_TARGET/lib/libstdc++.*
    run rm -rf $TOOLCHAIN_PATH/$ABI_CONFIGURE_TARGET/lib/*/libstdc++.*
    run rm -rf $TOOLCHAIN_PATH/$ABI_CONFIGURE_TARGET/include/c++
fi

# strip binaries to reduce final package size
run strip $TOOLCHAIN_PATH/bin/*$HOST_EXE
run strip $TOOLCHAIN_PATH/$ABI_CONFIGURE_TARGET/bin/*$HOST_EXE
run strip $TOOLCHAIN_PATH/libexec/gcc/*/*/cc1$HOST_EXE
run strip $TOOLCHAIN_PATH/libexec/gcc/*/*/cc1plus$HOST_EXE
run strip $TOOLCHAIN_PATH/libexec/gcc/*/*/collect2$HOST_EXE

dump "Done."
if [ -z "$OPTION_BUILD_OUT" ] ; then
    rm -rf $BUILD_OUT
fi
